%%% updated on May 17th, 2020
%%% This folder contains codes to replicate the paper Arellano (2008).
%%% Author: Ding Dong (HKUST); Yueqi Zhang (PHBS).
%%% Ding Dong, email: ding.dong@connect.ust.hk
%%% Yueqi Zhang, email: yueqi-zhang@pku.edu.cn
% 
% The code is written in Matlab. The code computes optimal policy functions, value function and bond price at equilibrium. 
% 
% MAIN.m is the main program, which calls on the following subroutines:
% 1. Parameter.m        set up parameters,
% 2. VFI.m              solve value function iteration problems
%                       (faster than VFI_2loops.m used in original paper)
% 3. Figure2.m          plot figure 2 in the paper using results from VFI                          
% 4. Figure3.m          plot figure 3 in the paper using results from VFI
% 5. Figure4.m          plot figure 4 in the paper using results from VFI
%
% The above routines call the following function:
% tauchen.m             function discretizing AR(1) process to Markov
% FunUc.m               utility function
% FunHy.m               output loss function


clear; 
clc; 
close all
%% Set Parameters
Parameter;

%% Set Grid
[y, Par.Pi] = tauchen(Par.mu,Par.rho,Par.sig,Par.ngridy);
       % y:  grid of log(income);  Par.Pi:  transition matrix      
grid.y = exp(y);
       % income grid
grid.b = linspace(Par.bmin,Par.bmax,Par.ngridb)';
       % debt grid
clear y;

%% Solve Steady State
%__________________________________________________________________________
%
% Special case: iid shock
%__________________________________________________________________________

% stationary distribution --> i.i.d transition matrix
mc = dtmc(Par.Pi);         
mu = repmat(asymptotics(mc),[Par.ngridy,1]);

% valeu function iteration
[Grid_iid]=VFI(mu,100,0,Par,grid);

% plot figure 2 in the paper
Figure2;

clear mc mu;
%__________________________________________________________________________
%
% General case: persistence shock
%__________________________________________________________________________

% value function iteration 
[Grid]=VFI(Par.Pi,Par.yhat,Par.theta,Par,grid);

% plot figure 3 and 4 in the paper
Figure3;
Figure4;


