%%% updated on May 15th, 2020
%%% Sovereign Default
%%% Ding Dong, email: ding.dong@connect.ust.hk
%%% Yueqi Zhang, email: yueqi-zhang@pku.edu.cn
%%%

clear; 
clc; 
close all
%% Set Parameters
Parameter;

%% Set Grid
[y, Par.Pi] = tauchen(Par.mu,Par.rho,Par.sig,Par.ngridy);
       % y:  grid of log(income);  Par.Pi:  transition matrix      
grid.y = exp(y);
       % income grid
grid.b = linspace(Par.bmin,Par.bmax,Par.ngridb)';
       % debt grid
clear y;

%% Steady State

% auxiliary matrix
Grid.Qb      = zeros(Par.ngridy,Par.ngridb);  % bond price
Grid.Vo      = zeros(Par.ngridy,Par.ngridb);  % value
Grid.Vd      = zeros(Par.ngridy,Par.ngridb);  % value of default
Grid.Vc      = zeros(Par.ngridy,Par.ngridb);  % value of repayment
Grid.Dft     = zeros(Par.ngridy,Par.ngridb);  % index of default decision

% utility at autarky
grid.hy      = FunHy(Par,grid.y);
grid.u_ydef  = FunUc(Par,grid.hy);               % vector: 1-d
Grid.U_ydef  = repmat(grid.u_ydef,1,Par.ngridb); % matrix: 2-d


tic_vfi = tic;

%--------------------------------------------------------------------------
% initial guess
%--------------------------------------------------------------------------

Vc  = rand(Par.ngridy,Par.ngridb);           % initial guess for Vc
Vd  = zeros(Par.ngridy,Par.ngridb);          % initial guess for Vd

Vo  = max(Vc,Vd);                            % initial guess for V

Dft = (Vc > Vd)*0 + (Vc <= Vd)*1;            % initial guess for Dft
     % no default: zeros(Par.ngridy,Par.ngridb);               
Qb  = ones(Par.ngridy,Par.ngridb)/(1+Par.r); % initial guess: risk-free

    % convergence criteria of inner loop
    dif_vc = 1;                              
    dif_vd = 1;
    tol_vc = 10^(-4);
    tol_vd = 10^(-4);

    v_iter = 0;
    
    %----------------------------------------------------------------------
    % inner loop: value function iteration
    %----------------------------------------------------------------------
    fprintf(1,'%12s%12s%12s\n','niter_v','dif_vc','dic_vd'); % Print out Iteration Header
    
    while dif_vc >=tol_vc  || dif_vd >=tol_vd   % inner loop start
        
        %------------------------------------------------------------------ 
        % value of default
        
        vo0    = Vo(:,grid.b==0);
        Vo0    = repmat(vo0,[1,Par.ngridb]);
               % value with no debt
        Vd_iter = (eye(Par.ngridy)-Par.beta*(1-Par.theta)*Par.Pi)\(Grid.U_ydef + Par.beta*Par.theta*Par.Pi*Vo0);

        %------------------------------------------------------------------
        % value of non-default
           
        Grid.Qb      = kron(Qb,ones(Par.ngridb,1));
                     % (ngridy,nrgidb')  -->  (ngridy*ngridb,ngridb')
                     
        % 3-dimension: (ngridy,ngridb,ngridb')
        GRID.Qb      = reshape(Grid.Qb,Par.ngridy,Par.ngridb,Par.ngridb);
        [GRID.Y,GRID.B,GRID.Bprime] = ndgrid(grid.y,grid.b,grid.b);
        GRID.Cons    = GRID.Y + GRID.B - GRID.Qb.*GRID.Bprime;
        GRID.U_cons  = (GRID.Cons>=0).*FunUc(Par,GRID.Cons)+ (GRID.Cons< 0).*(-10000);
                     
        % 3-dimension --> 2-dimension for grid search
        Grid.U_cons  = reshape(GRID.U_cons,Par.ngridy*Par.ngridb,Par.ngridb);
                     % (ngridy*ngridb,ngridb')
                     
        % 2-dimension --> 1-dimension in maximization: (ngridy*ngridb,1)
        [vc,idx_bprime] = max( Grid.U_cons + Par.beta*repmat(Par.Pi,[Par.ngridb,1])*Vo, [], 2); 
        bprime          = grid.b(idx_bprime);
                     
        % 1-dimension --> 2-dimension: (ngridy,ngridb)
        Vc_iter      = reshape(vc,Par.ngridy,Par.ngridb);
        Idx_bprime   = reshape(idx_bprime,Par.ngridy,Par.ngridb);
        Bprime       = reshape(bprime,Par.ngridy,Par.ngridb);
                     
               
        %% update value function until convergence  
        dif_vc = max(abs(Vc_iter(:)-Vc(:)));
        dif_vd = max(abs(Vd_iter(:)-Vd(:)));
        fprintf(1,'%12d%12.4g%12.4g\n',[v_iter,dif_vc,dif_vd]); % print out the iteration results
                
        Vd   = Vd_iter;
        Vc   = Vc_iter;
        Vo   = max(Vd,Vc);
        Dft  = (Vd < Vc)*0 + (Vd >= Vc)*1;
        
        % Grid.Edp: expected default probability
        for yy = 1: Par.ngridy
            for bb = 1:Par.ngridb
                Grid.Edp(yy,bb) = Par.Pi(yy,:)*Dft(:,Idx_bprime(yy,bb));
            end
        end
        
        v_iter = v_iter +1; 
        
    end     % inner loop end
    
%% update bond prices Qb function until convergence

Qb_iter = (ones(Par.ngridy,Par.ngridb)- Grid.Edp)/(1+Par.r);

    
toc_vfi = toc(tic_vfi);
